@@ -26,6 +26,16 @@ BEDPEFile <- function(resource) {
     new("BEDPEFile", resource = resource)
 }
 
+setClass("NarrowPeakFile", contains = "BEDFile")
+NarrowPeakFile <- function(resource) {
+  new("NarrowPeakFile", resource = resource)
+}
+
+setClass("BroadPeakFile", contains = "BEDFile")
+BroadPeakFile <- function(resource) {
+  new("BroadPeakFile", resource = resource)
+}
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Export
 ###
@@ -106,7 +116,8 @@ setMethod("export", c("GenomicRanges", "BEDFile"),
                 lastStart <- lastBlock(blockStarts)
                 if (any(df[[2]] + lastSize + lastStart != df[[3]]) ||
                     any(sub(",.*", "", blockStarts) != "0"))
-                  stop("blocks must span entire feature")
+                    stop("blocks must span entire feature,",
+                         " and be within its bounds")
                 blockCount <- elementNROWS(object$blocks)
                 if (!is.null(object$blockCount))
                   if (!identical(blockCount, as.integer(object$blockCount)))
@@ -291,6 +302,17 @@ setMethod("import", "BEDFile",
               pushBack(line, con)
               colsInFile <- seq_len(length(strsplit(line, "\\s+")[[1]]))
               presentNames <- bedNames[colsInFile]
+              lacksNames <- is.null(names(extraCols)) ||
+                  any(names(extraCols) == "") ||
+                  any(is.na(names(extraCols)))
+              if (length(extraCols) > 0L && lacksNames)
+                  stop("'extraCols' must have valid names")
+              reservedNames <- c(names(bedClasses), "thick", "blocks")
+              conflictingNames <- intersect(reservedNames, names(extraCols))
+              if (length(conflictingNames) > 0L) {
+                  stop("'names(extraCols)' contains reserved name(s): ",
+                       paste0("'", conflictingNames, "'", collapse=", "))
+              }
               tail(presentNames, length(extraCols)) <- names(extraCols)
               bedNames <- presentNames
               presentClasses <- bedClasses[colsInFile]
@@ -358,6 +380,38 @@ setMethod("colClasses", "BEDFile", function(x) {
                 blockSizes="character", blockStarts="character")
           })
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### narrowPeak support
+###
+setGeneric("import.narrowPeak", function(con, ...) standardGeneric("import.narrowPeak"))
+
+setMethod("import.narrowPeak", "ANY", function(con, ...) {
+    import(con, format = "narrowPeak", ...)
+
+})
+
+setMethod("import", "NarrowPeakFile", function(con, ...) {
+    callNextMethod(con = con,
+                   extraCols = c(signalValue = "numeric", pValue = "numeric", qValue = "numeric", peak = "integer"),
+                   ... )
+})
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### broadPeak support
+###
+setGeneric("import.broadPeak", function(con, ...) standardGeneric("import.broadPeak"))
+
+setMethod("import.broadPeak", "ANY", function(con, ...) {
+    import(con, format = "broadPeak", ...)
+
+})
+
+setMethod("import", "BroadPeakFile", function(con, ...) {
+    callNextMethod(con = con,
+                   extraCols = c(signalValue = "numeric", pValue = "numeric", qValue = "numeric"),
+                   ... )
+})
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### BED15 (Microarray) Support
 ###
@@ -441,7 +495,7 @@ setMethods("export",
 setClass("Bed15TrackLine",
          representation(expStep = "numeric", expScale = "numeric",
                         expNames = "character_OR_NULL"),
-         prototype(expStep = 0.5, expScale = 3.0), 
+         prototype(expStep = 0.5, expScale = 3.0),
          contains = "BasicTrackLine") # not sure which fields work
 
 setAs("Bed15TrackLine", "character",
